@@ -1,6 +1,7 @@
-from . import constants, retrieve
+from __future__ import annotations
+from . import constants, retrieve, t
 
 
-def load(doc):
+def load(doc: t.SpecT) -> None:
     code = retrieve.retrieveBoilerplateFile(doc, "bs-extensions", allowLocal=constants.executeCode)
     exec(code, globals())
